<?php

namespace Drupal\image_auto_alt_filter\Plugin\Filter;

use Drupal\Component\Utility\Xss;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\filter\FilterProcessResult;
use Drupal\filter\Plugin\FilterBase;
use DonatelloZa\RakePlus\RakePlus;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Controller\TitleResolver;

/**
 * Provides a filter to add ALT tag to images.
 *
 * @Filter(
 *   id = "image_auto_alt",
 *   title = @Translation("Image auto ALT filter"),
 *   type = Drupal\filter\Plugin\FilterInterface::TYPE_TRANSFORM_REVERSIBLE,
 *   settings = {
 *     "alt_method" = "keywords",
 *     "words_length" = "10",
 *     "extract_keywords_lng" = "en_US"
 *   }
 * )
 */
class ImageAutoAlt extends FilterBase implements ContainerFactoryPluginInterface {

  /**
   * The current Request object.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * The route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * The title resolver.
   *
   * @var \Drupal\Core\Controller\TitleResolver
   */
  protected $titleResolver;

  /**
   * Constructs a new ImageAutoAlt.
   *
   * @param array $configuration
   *   Configuration.
   * @param string $plugin_id
   *   Plugin ID.
   * @param mixed $plugin_definition
   *   Definition.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   Current request.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match service.
   * @param \Drupal\Core\Controller\TitleResolver $title_resolver
   *   Title resolver service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, Request $request, RouteMatchInterface $route_match, TitleResolver $title_resolver) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->request = $request;
    $this->routeMatch = $route_match;
    $this->titleResolver = $title_resolver;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('request_stack')->getCurrentRequest(),
      $container->get('current_route_match'),
      $container->get('title_resolver'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form['alt_method'] = [
      '#type' => 'select',
      '#title' => $this->t('Choose method for adding ALT.'),
      '#options' => [
        'page_title' => $this->t('Page title'),
        'keywords' => $this->t('Extract keywords from text'),
      ],
      '#default_value' => $this->settings['alt_method'],
    ];
    $form['words_length'] = [
      '#type' => 'number',
      '#title' => $this->t('Minimum length of your keyword phrases.'),
      '#min' => 1,
      '#default_value' => $this->settings['words_length'],
      '#field_suffix' => $this->t('characters'),
      '#description' => $this->t('Minimum keyword length in case of Extract keywords from text method'),
    ];
    $form['extract_keywords_lng'] = [
      '#type' => 'select',
      '#title' => $this->t('Language code for stop words.'),
      '#options' => [
        'ar_AE' => $this->t('Arabic'),
        'pt_BR' => $this->t('Brazilian Portuguese/português do Brasil'),
        'en_US' => $this->t('English US'),
        'pt_PT' => $this->t('European Portuguese/português europeu'),
        'fr_FR' => $this->t('French/le français'),
        'de_DE' => $this->t('German (Germany)/Deutsch (Deutschland)'),
        'pl_PL' => $this->t('Polish/język polski'),
        'es_AR' => $this->t('Spanish/español'),
      ],
      '#default_value' => $this->settings['extract_keywords_lng'],
      '#description' => $this->t('Stop words predefined list.'),
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function process($text, $langcode) {
    return new FilterProcessResult($this->proccessImages($text, $this));
  }

  /**
   * {@inheritdoc}
   */
  public function tips($long = FALSE) {
    return $this->t('Automatically add ALT tag to images.');
  }

  /**
   * Filter function.
   */
  private function proccessImages(string $text, $filter): string {
    $matches = [];
    $img_alt_pagetitle = '';
    $result2 = preg_match_all('/<img[^>]+>/i', $text, $matches);
    if ($this->settings['alt_method'] == 'keywords') {
      $keywords_phrases = RakePlus::create(Xss::filter(strip_tags($text)), $this->settings['extract_keywords_lng'], $this->settings['words_length'])
        ->get();
      if ($result2 > count($keywords_phrases)) {
        $newArray = [];
        while (count($newArray) <= $result2) {
          $newArray = array_merge($newArray, $keywords_phrases);
        }
        $keywords_phrases = array_slice($newArray, 0, $result2);
      }
    }
    else {
      $img_alt_pagetitle = $this->titleResolver
        ->getTitle($this->request, $this->routeMatch->getRouteObject());
    }
    if ($result2) {
      $image_attributes = [];
      foreach ($matches[0] as $key1 => $img_tag) {
        preg_match_all('/([\w\-]+)[ ]*=[ ]*"([^"]*)"/i', $img_tag, $final_matches);
        foreach ($final_matches[1] as $final_matches_look_alt) {
          $image_attributes[] = $final_matches_look_alt;
        }
        if (in_array("alt", $image_attributes) && !$final_matches[2][0]) {
          if ($filter->settings['alt_method'] == 'keywords') {
            if (isset($keywords_phrases[$key1])) {
              $html = preg_replace('/alt=["].*?["]/', 'alt="' . trim($keywords_phrases[$key1]) . '"', $img_tag);
            }
          }
          else {
            $html = preg_replace('/alt=["].*?["]/', 'alt="' . trim($img_alt_pagetitle) . '"', $img_tag);
          }
          if (isset($html)) {
            $filtered_text = str_replace($img_tag, $html, $text);
            if ($filtered_text) {
              $text = $filtered_text;
            }
          }
        }
        if (!in_array("alt", $image_attributes)) {
          if ($filter->settings['alt_method'] == 'keywords') {
            if (isset($keywords_phrases[$key1])) {
              $html = preg_replace('/<img /', '<img alt="' . trim($keywords_phrases[$key1]) . '"', $img_tag);
            }
          }
          else {
            $html = preg_replace('/<img /', '<img alt="' . trim($img_alt_pagetitle) . '"', $img_tag);
          }
          if (isset($html)) {
            $filtered_text = str_replace($img_tag, $html, $text);
            if ($filtered_text) {
              $text = $filtered_text;
            }
          }
        }
        unset($image_attributes);
        unset($final_matches);
      }
    }
    return $text;
  }

}
